"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
class Node {
    constructor(name) {
        this.data = new Map();
        this.inputs = new Map();
        this.outputs = new Map();
        this.controls = new Map();
        this.position = [0.0, 0.0];
        this.name = name;
        this.id = Node.nextId();
    }
    static nextId() {
        if (!this.latestId) {
            this.latestId = 1;
        }
        else {
            this.latestId++;
        }
        return this.latestId;
    }
    static resetId() {
        this.latestId = 0;
    }
    add(map, item, key) {
        if (map.has(item.key)) {
            throw new Error(errors_1.EngineError.ItemExistsOnThisNode);
        }
        if (item[key] !== null) {
            throw new Error(errors_1.EngineError.ItemExistsOnSomeNode);
        }
        item[key] = this;
        map.set(item.key, item);
    }
    addControl(control) {
        this.add(this.controls, control, 'parent');
        return this;
    }
    removeControl(control) {
        control.parent = null;
        this.controls.delete(control.key);
    }
    addInput(input) {
        this.add(this.inputs, input, 'node');
        return this;
    }
    removeInput(input) {
        input.removeAllConnections();
        input.node = null;
        this.inputs.delete(input.key);
    }
    addOutput(output) {
        this.add(this.outputs, output, 'node');
        return this;
    }
    removeOutput(output) {
        output.removeAllConnections();
        output.node = null;
        this.outputs.delete(output.key);
    }
    getConnections() {
        const connections = [];
        for (const input of this.inputs.values()) {
            for (const connection of input.connections) {
                connections.push(connection);
            }
        }
        for (const output of this.outputs.values()) {
            for (const connection of output.connections) {
                connections.push(connection);
            }
        }
        return connections;
    }
    toJSON() {
        const data = {};
        for (const entry of this.data) {
            data[entry[0]] = entry[1];
        }
        const inputs = {};
        for (const entry of this.inputs) {
            inputs[entry[0]] = entry[1].toJSON();
        }
        const outputs = {};
        for (const entry of this.outputs) {
            outputs[entry[0]] = entry[1].toJSON();
        }
        return {
            'id': this.id,
            'name': this.name,
            'data': data,
            'inputs': inputs,
            'outputs': outputs,
            'position': this.position,
        };
    }
    static fromJSON(json) {
        const node = new Node(json.name);
        node.id = json.id;
        node.data = new Map();
        Object.entries(json.data).forEach(([key, value]) => node.data.set(key, value));
        const [x, y] = json.position;
        node.position = [x, y];
        Node.latestId = Math.max(node.id, Node.latestId);
        return node;
    }
    update() { }
}
exports.Node = Node;
Node.latestId = 0;
//# sourceMappingURL=node.js.map